------------------------------------------------------------------------------
-- Warning: API for units modding is still not finalized and can be changed
--          in the future !!!
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Loading frames from png file by specifying coordinates and size of each 
-- subframe and passing them to png_image_fragment() function
------------------------------------------------------------------------------

local frames = {}

for row = 0, 7 do
    for col = 0, 7 do
        table.insert(frames, png_image_fragment("$(extension)/metapod.png",
            1 + col * 33, 1 + row * 41, 32, 40))
    end
end

local crouching_frames = {}

for row = 0, 1 do
    for col = 0, 7 do
        table.insert(crouching_frames, png_image_fragment("$(extension)/metapod.png",
            298 + col * 33, 1 + row * 41, 32, 40))
    end
end

local falling_frames = {}

for row = 0, 7 do
    table.insert(falling_frames, png_image_fragment("$(extension)/metapod.png",
         265, 1 + row * 41, 32, 40))
end

------------------------------------------------------------------------------
-- This function renders unit sprite image by probably combining several
-- frames together and drawing them all on img object which is
-- passed to this function as the first argument.
--
-- img    - target sprite object, it supports 'draw' method, which
--          accepts 3 arguments: x, y coordinates and a bitmap
--          object (loaded from png or pck file for example)
-- dir    - direction the soldier is facing (0, 1, 2, 3, 4, 5, 6, 7)
-- phase  - animation phase (0 - no animation, static image, 1-8 sequential
--          frames)
--
-- info   - table with some additional information, currently it can contain
--          the following properties:
--            "hand_object_image" - contains bitmap with the weapon image
--            "aimed_hand_object_image" - contains bitmap with the weapon image rotated 90 degrees
--            "is_crouching"      - boolean flag, shows that the soldier is crouching
--            "is_falling"        - boolean flag, indicates request for death or stun animation
--            "is_flying"         - boolean flag, indicates that the soldier is flying
------------------------------------------------------------------------------

local function sprite_renderer(img, dir, phase, info)

    local dframes = 0
    
    if info.is_falling then
        img:draw(0, 0, falling_frames[phase])
        return
    end
    if info.hand_object_image then
        dframes = 8
    end
    if not info.is_crouching then
        if phase ~= 0 then
	    if (dir == 0) then
                img:draw(0, 0, frames[16 + dframes + phase])
	    elseif (dir == 7) then
                img:draw(0, 0, frames[32 + dframes + phase])
	    elseif (dir == 6) then
                img:draw(0, 0, frames[48 + dframes + phase])
            else
                img:draw(0, 0, frames[8 - dir + dframes])
            end
        else
            img:draw(0, 0, frames[8 - dir + dframes])
        end
    else
        img:draw(0, 0, crouching_frames[8 - dir + dframes])   
    end
end

AddUnit {
    name = "Metapod",
    pMap  = sprite_renderer,
    pInv = png_image_fragment("$(extension)/metapod_inv.png", 0, 0, 80, 100)
}
